<?php

declare(strict_types=1);

namespace mirzaev\calculator\models\filters;

use mirzaev\calculator\models\filters\import_filter as filter;
use mirzaev\calculator\models\metals_model as metals;

use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;

/**
 * Фильтр импорта для документов от "МЕТАЛЛСЕРВИС"
 *
 * @package mirzaev\calculator\models\filters
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class import_filter_mc implements filter
{
    /**
     * Название металла
     */
    public string $metal = '';

    /**
     * Чтение ячейки
     *
     * @param [type] $columnAddress
     * @param [type] $row
     * @param string $worksheetName
     *
     * @return void
     */
    public function readCell($columnAddress, $row, $worksheetName = '')
    {
        return false;
    }

    /**
     * Фильтрация и запись прочитанных данных из EXCEL-документа
     *
     * @param int $supply Идентификатор поставки (для записи в БД)
     * @param Worksheet $spreadsheet Таблица с прочитанными данными
     * @param array &$errors Журнал ошибок
     *
     * @return int Количество записанных строк
     */
    public function write(int $supply, Worksheet $spreadsheet, array &$errors = []): int
    {
        // Инициализация счётчика записанных данных
        $writed = 0;

        for ($row = 1; $row <= $spreadsheet->getHighestRow(); ++$row) {
            // Перебор строк

            // Инициализация переменных
            $width = $height = $length = $piece = $ton = null;

            for ($col = 'A'; $col <= $spreadsheet->getHighestColumn(); ++$col) {
                // Перебор столбцов

                if ($col === 'B') {
                    // Колонка с размером

                    // Чтение из ячейки
                    $size = $spreadsheet->getCell($col . $row)->getValue();
                } else if ($col === 'C') {
                    // Колонка с маркой

                    // Чтение из ячейки
                    $mark = $spreadsheet->getCell($col . $row)->getValue();
                } else if ($col === 'E') {
                    // Колонка с ценой за тонну

                    // Чтение из ячейки
                    $ton = (int) round((float) $spreadsheet->getCell($col . $row)->getCalculatedValue());
                } else if ($col === 'I') {
                    // Колонка с ценой за штуку

                    // Чтение из ячейки
                    $piece = (int) round((float) $spreadsheet->getCell($col . $row)->getCalculatedValue());
                }
            }

            // Пропуск пустых строк
            if (empty($size) || empty($mark) || empty($ton) || empty($piece)) continue;

            // Извлечение размеров
            preg_match_all('/[\d,\.]+/', $size, $matches);

            // Запись размеров
            $length = (float) str_replace(',', '.', str_replace('.', '', (string) $matches[0][0]));
            $width = (float) str_replace(',', '.', str_replace('.', '', (string) $matches[0][1]));
            $height = (float) str_replace(',', '.', str_replace('.', '', (string) $matches[0][2]));

            // Запись металла
            metals::write($supply, $this->metal, $mark, $width, $height, $length, $piece, $ton, $errors) ? ++$writed : null;
        }

        return $writed;
    }
}
