<?php

declare(strict_types=1);

namespace mirzaev\calculator\models\filters;

use mirzaev\calculator\models\filters\import_filter as filter;
use mirzaev\calculator\models\metals_model as metals;

use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;

/**
 * Фильтр импорта для документов от неизвестного поставщика под номером 1
 *
 * @package mirzaev\calculator\models\filters
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class import_filter_noname_1 implements filter
{
    /**
     * Название металла
     */
    public string $metal = '';

    /**
     * Чтение ячейки
     *
     * @param [type] $columnAddress
     * @param [type] $row
     * @param string $worksheetName
     *
     * @return void
     */
    public function readCell($columnAddress, $row, $worksheetName = '')
    {
        return false;
    }

    /**
     * Фильтрация и запись прочитанных данных из EXCEL-документа
     *
     * @param int $supply Идентификатор поставки (для записи в БД)
     * @param Worksheet $spreadsheet Таблица с прочитанными данными
     * @param array &$errors Журнал ошибок
     *
     * @return int Количество записанных строк
     */
    public function write(int $supply, Worksheet $spreadsheet, array &$errors = []): int
    {
        // Инициализация счётчика записанных данных
        $writed = 0;

        for ($row = 1; $row <= $spreadsheet->getHighestRow(); ++$row) {
            // Перебор строк

            // Инициализация переменных
            $width = $height = $length = $piece = $ton = null;

            for ($col = 1; $col <= coordinate::columnIndexFromString($spreadsheet->getHighestColumn()); ++$col) {
                // Перебор столбцов

                if ($col === 1) {
                    // Колонка с металлом

                    // Чтение из ячейки
                    $metal = $spreadsheet->getCellByColumnAndRow($col, $row)->getValue();
                } else if ($col === 14) {
                    // Колонка с коэффициентом

                    // Чтение из ячейки
                    $coefficient = (float) $spreadsheet->getCellByColumnAndRow($col, $row)->getValue();
                } else if ($col === 15) {
                    // Колонка с ценой

                    // Чтение из ячейки
                    $piece = (int) $spreadsheet->getCellByColumnAndRow($col, $row)->getValue();
                }
            }

            // Пропуск пустых строк
            if (empty($metal)) continue;

            // Извлечение размеров
            preg_match_all('/[\sxх]([\d]+[,\.]?[\d]?)/', $metal, $matches);

            // Запись размеров
            $length = (float) str_replace(',', '.', str_replace('.', '', (string) $matches[1][0]));
            $width = (float) str_replace(',', '.', str_replace('.', '', (string) $matches[1][1]));
            $height = (float) str_replace(',', '.', str_replace('.', '', (string) $matches[1][2]));

            // Извлечение марки металла
            preg_match_all('/\d+\s+(.*)$/', $metal, $matches);

            // Запись марки металла
            $mark = trim((string) $matches[1][0]);

            if ($length > 0 && $width > 0 && $height > 0 && !empty($mark) && $piece > 0 && $coefficient > 0) {
                // Найдены все размеры, марка, цена и коэффициент

                // Цена за тонну
                $ton = (int) round((float) ($piece / $coefficient * 1000));

                // Запись металла
                metals::write($supply, $this->metal, $mark, $width, $height, $length, $piece, $ton, $errors) ? ++$writed : null;
            }
        }

        return $writed;
    }
}
