<?php

declare(strict_types=1);

namespace mirzaev\calculator\models;

use mirzaev\calculator\models\accounts_model as accounts;
use mirzaev\calculator\models\filters\import_filter as filter;

use PhpOffice\PhpSpreadsheet\Reader\Xls as reader;

use pdo;
use exception;

/**
 * Модель поставки
 *
 * @package mirzaev\calculator\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class supplies_model extends core
{
    /**
     * Запись в базу данных
     *
     * @param string $file Путь до файла
     * @param filter|null $filter Фильтр строк
     * @param array &$errors Журнал ошибок
     *
     * @return bool Статус записи
     */
    public static function write(string $file, ?filter $filter = null, array &$errors = []): bool
    {
        try {
            if (file_exists($file) && $account = accounts::account($errors)) {
                // Найден файл и инициализирован аккаунт

                // Инициализация обработчика документа
                $reader = new reader();

                // Инициализация фильтра строк
                if (isset($filter)) $reader->setReadFilter($filter);

                // Чтение файла
                $spreadsheet = $reader->load($file)->getActiveSheet();

                // Инициализация запроса
                $request = static::$db->prepare("INSERT INTO `supplies` (`account`) VALUES (:account)");

                // Инициализация параметров
                $params = [
                    ':account' => $account['id'],
                ];

                // Отправка запроса
                $request->execute($params);

                // Запись идентификатора поставки
                $supply = (int) (static::$db->lastInsertId() ?? throw new exception('Не удалось найти идентификатор записанной поставки'));

                // Запись в базу данных
                $filter->write($supply, $spreadsheet, $errors);

                return true;
            } else {
                // Не найден файл

                throw new exception("Файл не найден: $file");
            }
        } catch (exception $e) {
            // Запись в журнал ошибок
            $errors[] = [
                'text' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'stack' => $e->getTrace()
            ];
        }

        return false;
    }
}
